import os

DEFAULT_TIMOUT_PAGE_LOAD = int(os.environ.get("SOP_TIMOUT_PAGE_LOAD", 3))
"""Default timeout while loading page
or moving from a menu to an other.

This default timeout for things that should
failed quickly as it should be fast as possible
and straight forward operations, loading list view
form view and so on.
"""

DEFAULT_TIMOUT_ACTION = int(os.environ.get("SOP_TIMOUT_ACTION", 5))
"""Default timeout while doing action that require
more time to compute something like generating report
genrating new invoice from sale order and loading
its form view...
"""



import email
import os
import time

import requests

MAIL_HOG = os.environ.get("MAIL_HOG", "http://127.0.0.1:8025")


def wait_email(
    kind,
    query,
    expected_total=1,
    timeout=15,
    timeout_message="email (kind: {kind}, {query}) wasn't found in the given time {timeout}s",
):
    """
    - mailhog **kind**: Kind of search: enum: [ from, to, containing ]
    - mailhog **query**: Search parameter: string
    - **expected_total**: assert expected number of document for given query
    - **timeout**: time in second to wait email
    - **timout_message**: raised TimeoutError message
    """
    start_time = time.perf_counter()
    while True:
        time.sleep(0.5)
        response = requests.get(MAIL_HOG + f"/api/v2/search?kind={kind}&query={query}")
        assert response.status_code == 200, "Fake SMTP service wasn't ready"
        data = response.json()
        if data["total"] == expected_total:
            return data
        if (time.perf_counter() - start_time) > timeout:
            raise TimeoutError(
                timeout_message.format(kind=kind, query=query, timeout=timeout)
            )


def pdfs_from_email(email_content):
    msg = email.message_from_bytes(email_content.get("Raw", {}).get("Data").encode())
    pdfs = {}
    for payload in msg.get_payload():
        if payload.get_content_type() == "application/pdf":
            pdfs[payload.get_filename()] = payload.get_payload()
    return pdfs



"""Generic Selenium elements which are not tied to odoo
"""
from typing import TYPE_CHECKING, Tuple

from selenium.common.exceptions import (
    ElementClickInterceptedException,
    StaleElementReferenceException,
)
from selenium.webdriver import ActionChains
from selenium.webdriver.support import expected_conditions as EC

if TYPE_CHECKING:
    from selenium import webdriver

Locator = Tuple[str, str]


class ClickElement(EC.element_to_be_clickable):
    """Try to click on an element until it works.

    This extend EC.element_to_be_clickable by catching ElementClickInterceptedException
    to make sure that there is no overlay stoping the click event propagation.

    usage:

    WebDriverWait(self.driver, timeout).until(
        ClickElement(A_LOCATOR)
    )
    """

    def __call__(self, driver: "webdriver"):
        try:
            element = super(ClickElement, self).__call__(driver)
            if element:
                ActionChains(element.parent).move_to_element(element).perform()
                element.click()
                return True
        except (ElementClickInterceptedException, StaleElementReferenceException):
            # we ignore if any overlay stop the event propagation
            return False
        return element


def expected_number_tabs(number: int):
    """An Expectation for checking how many tabs opened in browser."""

    def _predicate(driver):
        if len(driver.window_handles) == number:
            return number
        else:
            return False

    return _predicate


class TextToBeDifferentInElement:
    def __init__(self, locator: Locator, value: str):
        self.locator = locator
        self.value = value

    def __call__(self, driver: "webdriver"):
        element = driver.find_element(*self.locator)
        if not element:
            return False
        return element.text != self.value


class TextToBeEqual(object):
    """An expectation for checking if the given text property of the given
    object to be equal to the given expected value
    """

    def __init__(self, element: object, expected_value, property_attr="text"):
        self.element = element
        self.expected_value = expected_value
        self.property_attr = property_attr

    def __call__(self, driver):
        try:
            return self.expected_value == getattr(self.element, self.property_attr, "")
        except StaleElementReferenceException:
            return False



import pathlib

from setuptools import setup

HERE = pathlib.Path(__file__).parent


def parse_requirements(file):
    required = []
    with open(file) as f:
        for req in f.read().splitlines():
            if not req.strip().startswith("#"):
                required.append(req)
    return required


version = "0.1.0"
requires = parse_requirements("requirements.txt")
tests_requires = parse_requirements("requirements.tests.txt")
README = (HERE / "README.md").read_text()

setup(
    name="selenium-odoo-pages",
    version=version,
    description="A set of pages and elements used to interact "
    "with odoo pages and widget using the page selenium objects pattern",
    long_description=README,
    long_description_content_type="text/markdown",
    classifiers=[
        "Development Status :: 4 - Beta",
        "License :: OSI Approved :: Apache Software License",
        "Programming Language :: Python :: 3",
    ],
    author="Pierre Verkest",
    author_email="pierreverkest84@gmail.com",
    url="https://gitlab.com/micro-entreprise/selenium-odoo-pages",
    packages=["sop"],
    include_package_data=True,
    zip_safe=False,
    install_requires=requires,
    tests_require=requires + tests_requires,
    entry_points="""
    """,
)
